#include <p18f4550.h>
#include <delays.h>
#include "pins.h"       /* Pin definitions */
#include "constants.h"  /* Misc. constants */

/* Prototypes */
void SetScreen(void);
void SetMotor(void);
void SetBrains(void);

int Process(int);

void SetPin(int,int);

/* Boot code */
extern void _startup (void);
#pragma code _RESET_INTERRUPT_VECTOR = 0x001000
void _reset (void) {
    _asm goto _startup _endasm
}
#pragma code
/* End boot code */

/* Set the number of instructuions to be carried out here. */
#define INSTRUCTION_COUNT 9

/* This array lists the bots instuctions */
int instructions[INSTRUCTION_COUNT] = {
    INT_STOP,
    INT_LEFT,
    INT_RIGHT,
    INT_FORWARD,
    INT_STOP,
    INT_LEFT,
    INT_RIGHT,
    INT_BACK,
    INT_STOP
};

/* This array specifies the length of time for each instruction. */
int timing[INSTRUCTION_COUNT] = {
    3,
    1,
    1,
    1,
    3,
    2,
    2,
    1,
    1
};

void main(void) {
    int done = INSTRUCTION_COUNT;

    /* Set pin directions for this board */
    SetBrains();

    /* Main loop */
    while( done > 0 ) {
        done = Process(done);
    }

    /* When the sequence is done, stop */
    while(1);
}

/* Function to process instruction list */
int Process(int instruction) {
    int counter = 0;
    /* Fetch instruction to process */
    int current = INSTRUCTION_COUNT - instruction;

    /* Send command by setting the corresponding pin */
    SetPin( instructions[current], HIGH );

    /* Count ITU (Ivibot Time Units) They're about a second. */
    while(counter < timing[current] * 5) {
        Delay10KTCYx(0);
        counter++;
    }

    /* When the timer has run out, turn off the pin */
    SetPin( instructions[current], LOW );

    return --instruction;
}

/* Function to set pin values */
void SetPin( int pin, int setting ) {
    switch( pin ) {
        case INT_FORWARD:   FORWARD = setting;
        break;
        case INT_LEFT:      LEFT    = setting;
        break;
        case INT_RIGHT:     RIGHT   = setting;
        break;
        case INT_BACK:      BACK    = setting;
        break;
        case INT_STOP:      STOP    = setting;
        break;
    }
}

/* Set input and output pins needed for the screen board */
void SetScreen(void) {
    TRISD = 0x00;
    TRISB = 0xF8;
}

/* Set output pins needed for the brains board */
void SetBrains(void) {
    TRISD = 0x00;
    TRISB = 0x00;

    /* zero out control ports */
    FORWARD = LOW;
    LEFT    = LOW;
    RIGHT   = LOW;
    BACK    = LOW;
    STOP    = LOW;

    /* Needed under new configuration for reverse motor drive */
    TRISE = 0x00;
    TRISC = 0x00;
}

/* Set input and output pins needed for the motor board */
void SetMotor(void) {
    TRISD = 0xFF;
    TRISB = 0xFF;
    TRISE = 0x00;
    TRISC = 0x00;
}
