#include <p18f4550.h>
#include "delays.h"
#include "pins.h"

void SetScreen(void);
void SetMotor(void);
void SetBrains(void);
void SetMotorLeft(int);
void SetMotorRight(int);
void SetMotorFwd(int);
void SetMotorRev(int);

int RAMP_LEFT_UP, RAMP_LEFT_DOWN, RAMP_RIGHT_UP, RAMP_RIGHT_DOWN;

extern void _startup (void);
#pragma code _RESET_INTERRUPT_VECTOR = 0x001000
void _reset (void) {
    _asm goto _startup _endasm
}

#pragma code

void main(void) {
    RAMP_LEFT_UP = 0;
    RAMP_LEFT_DOWN = 0;
    RAMP_RIGHT_UP = 0;
    RAMP_RIGHT_DOWN = 0;

    SetMotor();

    MOTOR_RIGHT = 0;
    MOTOR_LEFT = 0;
    RAMP_LEFT_UP = 0;
    RAMP_LEFT_DOWN = 0;
    RAMP_RIGHT_UP = 0;
    RAMP_RIGHT_DOWN = 0;


    while(1) {
       
        if (LEFT) {
            MOTOR_LEFT      = HIGH;
            MOTOR_LEFT_R    = LOW;
            MOTOR_RIGHT     = LOW;
            MOTOR_RIGHT_R   = HIGH;
        } else if(RIGHT) {
            MOTOR_LEFT      = LOW;
            MOTOR_LEFT_R    = HIGH;
            MOTOR_RIGHT     = HIGH;
            MOTOR_RIGHT_R   = LOW;
        } else if(FORWARD) {
            MOTOR_LEFT      = HIGH;
            MOTOR_LEFT_R    = LOW;
            MOTOR_RIGHT     = HIGH;
            MOTOR_RIGHT_R   = LOW;
        } else if(BACK) {
            MOTOR_LEFT      = LOW;
            MOTOR_LEFT_R    = HIGH;
            MOTOR_RIGHT     = LOW;
            MOTOR_RIGHT_R   = HIGH;
        } else { /* Must be stop, but this also acts as safeguard */
            MOTOR_LEFT      = LOW;
            MOTOR_LEFT_R    = LOW;
            MOTOR_RIGHT     = LOW;
            MOTOR_RIGHT_R   = LOW;
        }
    
   }
}

void SetScreen(void) {
    TRISD = 0x00;
    TRISB = 0xF8;
}

void SetBrains(void) {
    TRISD = 0x00;
    TRISB = 0x00;
}

void SetMotor(void) {
    ADCON1 = 0x0F; // Disable analog input
    TRISD = 0xFF;
    TRISB = 0xFF;
    TRISE = 0x00;
    TRISC = 0x00;
}

void SetMotorLeft(int status) {
  if (status == 0) {
    MOTOR_LEFT = 0;
    RAMP_LEFT_UP = 1;
    RAMP_LEFT_DOWN = 0;
    return;
  }
  if (status) {
      MOTOR_LEFT = 1;
      return;
  }
  if (RAMP_LEFT_UP) {
    MOTOR_LEFT = 1;
    Delay10KTCYx(5);
    MOTOR_LEFT = 0;
    Delay10KTCYx(5);
    MOTOR_LEFT = 1;
    Delay10KTCYx(15);
    MOTOR_LEFT = 0;
    Delay10KTCYx(5);
  }
  MOTOR_LEFT = 1;
  RAMP_LEFT_UP = 0;
  RAMP_LEFT_DOWN = 1;
}

void SetMotorRight(int status) {
  if (status == 0) {
    MOTOR_RIGHT = 0;
    RAMP_RIGHT_UP = 1;
    RAMP_RIGHT_DOWN = 0;
    return;
}
  if (status == 1) {
   MOTOR_RIGHT = 1;
   return;
  }

  if (RAMP_RIGHT_UP) {
    MOTOR_RIGHT = 1;
    Delay10KTCYx(5);
    MOTOR_RIGHT = 0;
    Delay10KTCYx(5);
    MOTOR_RIGHT = 1;
    Delay10KTCYx(15);
    MOTOR_RIGHT = 0;
    Delay10KTCYx(5);
  }
  MOTOR_RIGHT = 1;
  RAMP_RIGHT_UP = 0;
  RAMP_RIGHT_DOWN = 1;
}


void SetMotorFwd(int status) {
  if (status == 0) {
    MOTOR_RIGHT = 0;
    MOTOR_LEFT = 0;
    RAMP_RIGHT_UP = 1;
    RAMP_LEFT_UP = 1;
    RAMP_RIGHT_DOWN = 0;
    RAMP_LEFT_DOWN = 0;
    return;
}
  if (RAMP_RIGHT_UP ||  RAMP_LEFT_UP) {
    MOTOR_RIGHT = 1;
    MOTOR_LEFT = 1;
    Delay10KTCYx(5);
    MOTOR_RIGHT = 0;
    MOTOR_LEFT = 0;
    Delay10KTCYx(5);
    MOTOR_RIGHT = 1;
    MOTOR_LEFT = 1;
    Delay10KTCYx(15);
    MOTOR_RIGHT = 0;
    MOTOR_LEFT = 0;
    Delay10KTCYx(5);
  }
  MOTOR_RIGHT = 1;
  MOTOR_LEFT = 1;
  RAMP_RIGHT_UP = 0;
  RAMP_LEFT_UP = 0;
  RAMP_RIGHT_DOWN = 1;
  RAMP_LEFT_DOWN = 1;

}

